/*
Copyright 2016 S7connector members (github.com/s7connector)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package com.cnbm.s7.s7connector.impl;

import com.cnbm.s7.s7connector.api.DaveArea;
import com.cnbm.s7.s7connector.api.S7Connector;
import com.cnbm.s7.s7connector.exception.S7CheckResultException;
import com.cnbm.s7.s7connector.impl.nodave.Nodave;
import com.cnbm.s7.s7connector.impl.nodave.S7Connection;
import com.cnbm.s7.s7connector.type.PlcVar;
import com.cnbm.s7.s7connector.type.TransportSize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Base-Connection for the S7-PLC Connection Libnodave:
 * http://libnodave.sourceforge.net/
 *
 * @author Thomas Rudin
 */
public abstract class S7BaseConnection implements S7Connector {

	private static final Logger logger = LoggerFactory.getLogger(S7BaseConnection.class);


	/** The Constant MAX_SIZE. */
	//private static final int MAX_SIZE = 96;     //原版96 ;; ioserver 127  ;;
	private static final int MAX_SIZE = 212;     //S7-1200-maxReadBytes = 212  ;;  S7-1500-maxReadBytes = 452

	/** The Constant PROPERTY_AREA. */
	public static final String PROPERTY_AREA = "area";

	/** The Constant PROPERTY_AREANUMBER. */
	public static final String PROPERTY_AREANUMBER = "areanumber";

	/** The Constant PROPERTY_BYTES. */
	public static final String PROPERTY_BYTES = "bytes";

	/** The Constant PROPERTY_OFFSET. */
	public static final String PROPERTY_OFFSET = "offset";

	/**
	 * Checks the Result.
	 *
	 * @param libnodaveResult
	 *            the libnodave result
	 */
	public static void checkResult(final int libnodaveResult) {
		if (libnodaveResult != Nodave.RESULT_OK) {
			final String msg = Nodave.strerror(libnodaveResult);
//			throw new IllegalArgumentException("Result: " + msg);
			logger.info("checkResult 失败，errValue:"+libnodaveResult+", errMsg: "+msg);
			throw new S7CheckResultException("errMsg : "+msg);
		}
	}

	/**
	 * Dump data
	 *
	 * @param b
	 *            the byte stream
	 */
	protected static void dump(final byte[] b) {
		for (final byte element : b) {
			System.out.print(Integer.toHexString(element & 0xFF) + ",");
		}
	}

	/** The dc. */
	private S7Connection dc;

	/**
	 * Initialize the connection
	 *
	 * @param dc
	 *            the connection instance
	 */
	protected void init(final S7Connection dc) {
		this.dc = dc;
	}

	/** {@inheritDoc} */
	@Override
	public synchronized byte[] read(final DaveArea area, final int areaNumber, final int bytes, final int offset) {
		if (bytes > MAX_SIZE) {
			final byte[] ret = new byte[bytes];
			//注意这里 嵌套了 递归，让无限递归去解决 MAX_SIZE的问题。
			final byte[] currentBuffer = this.read(area, areaNumber, MAX_SIZE, offset);
			System.arraycopy(currentBuffer, 0, ret, 0, currentBuffer.length);

			final byte[] nextBuffer = this.read(area, areaNumber, bytes - MAX_SIZE, offset + MAX_SIZE);
			System.arraycopy(nextBuffer, 0, ret, currentBuffer.length, nextBuffer.length);

			return ret;
		} else {
			//接收结果的 容器，传进去指针
			final byte[] buffer = new byte[bytes];
			final int ret = this.dc.readBytes(area, areaNumber, offset, bytes, buffer);

			//判断结果

			if(ret == Nodave.RESULT_OK){
				return buffer;
			}else {
				final String msg = Nodave.strerror(ret);
				logger.info("read checkResult 失败，errValue:"+ret+", errMsg: "+msg);
				throw new S7CheckResultException("errMsg : "+msg);
			}

		}
	}


//	private Integer checkResult(final int libnodaveResult) {
//		if (libnodaveResult != Nodave.RESULT_OK) {
//			final String msg = Nodave.strerror(libnodaveResult);
////			throw new IllegalArgumentException("Result: " + msg);
//			logger.info("checkResult 失败，errValue:"+libnodaveResult+", errMsg: "+msg);
//			throw new S7CheckResultException("errMsg : "+msg);
//		}
//	}



	@Override
	public synchronized byte[] read(final DaveArea area, final int areaNumber, final int bytes, final int offset, int bitOffset, TransportSize transportSize) {
		if(bitOffset==0){
			return read(area,areaNumber,bytes,offset);
		}

		if (bytes > MAX_SIZE) {
			final byte[] ret = new byte[bytes];
			//注意这里 嵌套了 递归，让无限递归去解决 MAX_SIZE的问题。
			final byte[] currentBuffer = this.read(area, areaNumber, MAX_SIZE, offset,bitOffset,transportSize);
			System.arraycopy(currentBuffer, 0, ret, 0, currentBuffer.length);

			final byte[] nextBuffer = this.read(area, areaNumber, bytes - MAX_SIZE, offset + MAX_SIZE,bitOffset,transportSize);
			System.arraycopy(nextBuffer, 0, ret, currentBuffer.length, nextBuffer.length);

			return ret;
		} else {
			//接收结果的 容器，传进去指针
			final byte[] buffer = new byte[bytes];
			final int ret = this.dc.readBytes(area, areaNumber, offset,bitOffset, bytes, buffer,transportSize);

			//thr resultException
			if(ret == Nodave.RESULT_OK){
				return buffer;
			}else {
				final String msg = Nodave.strerror(ret);
				logger.info("read checkResult 失败，errValue:"+ret+", errMsg: "+msg);
				throw new S7CheckResultException("errMsg : "+msg);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public synchronized void write(final DaveArea area, final int areaNumber, final int offset, final byte[] buffer) {
		if (buffer.length > MAX_SIZE) {
			// Split buffer
			final byte[] subBuffer = new byte[MAX_SIZE];
			final byte[] nextBuffer = new byte[buffer.length - subBuffer.length];

			System.arraycopy(buffer, 0, subBuffer, 0, subBuffer.length);
			System.arraycopy(buffer, MAX_SIZE, nextBuffer, 0, nextBuffer.length);

			this.write(area, areaNumber, offset, subBuffer);
			this.write(area, areaNumber, offset + subBuffer.length, nextBuffer);
		} else {
			// Size fits
			final int ret = this.dc.writeBytes(area, areaNumber, offset, buffer.length, buffer);
			// Check return-value
			checkResult(ret);
		}
	}

	@Override
	public synchronized void write(final DaveArea area, final int areaNumber, final int byteOffset, final int bitOffset, final byte[] buffer, PlcVar var) {
		if(bitOffset==0){
			write(area,areaNumber,byteOffset,buffer);
			return;
		}
		if (buffer.length > MAX_SIZE) {
			// Split buffer
			final byte[] subBuffer = new byte[MAX_SIZE];
			final byte[] nextBuffer = new byte[buffer.length - subBuffer.length];

			System.arraycopy(buffer, 0, subBuffer, 0, subBuffer.length);
			System.arraycopy(buffer, MAX_SIZE, nextBuffer, 0, nextBuffer.length);

			this.write(area, areaNumber, byteOffset,bitOffset, subBuffer,var);
			this.write(area, areaNumber, byteOffset + subBuffer.length,bitOffset, nextBuffer,var);
		} else {
			// Size fits
			final int ret = this.dc.writeBytes(area, areaNumber, byteOffset,bitOffset, buffer.length, buffer,var);
			// Check return-value
			checkResult(ret);
		}
	}
}
